#author = "Kang Yang"
#copyright = "Copyright 2018, Nanjing Greenland Hydrology Group"
#license = "GPL"
#version = "0.1"
#maintainer = "Kang Yang"
#email = "kangyang@nju.edu.cn"
#status = "Production"
#description = "create flow direction, flow accumulation, flow path distance datasets for different-resolution DEMs"


def resample_dem(demFile, resolutionList):
    demFiles = [demFile]
    for resolution in resolutionList:
        output_demFile = demFile.split('.')[0] + '_' + str(resolution) + '.tif'
        demFiles.append(output_demFile)
        arcpy.Resample_management(demFile, output_demFile, str(resolution),  'NEAREST')
        print 'create ' + str(resolution) + ' m DEM done'
    return demFiles


import arcpy,os,sys
from arcpy import env
from arcpy.sa import *

env.overwriteOutput=True


arcpy.CheckOutExtension("Spatial")
env.workspace=r'D:\Meltwater_routing_modeling_new'
env.extent="MAXOF"    #this sentence is important to keeep the same extent during processing
outputPath = r'D:\Meltwater_routing_modeling_new\simultaneous_image_outputs'
if os.path.exists(outputPath)==False:
        os.mkdir(outputPath)

# Inputs
input_demFile = 'IQ1_SETSM_WV02_20140707.tif'
resolutionList = [5,10,30,90]


#erase the depressions that located in the erase area and keep all the other depressions as potential catchment outlets
eraseAreaFile = r'D:\Meltwater_routing_modeling_new\IQ1_eraseArea.shp'

#use key point to find the goal catchment
keypointFile = r'D:\Meltwater_routing_modeling_new\IQ1_keypoint.shp'



# 0, only keep the depression that contain key point, fill all the other depressions
# 1, use erase area file to fill the depressions that intersect with erase area (commonly the upstream depressions in the goal IDC) and keep all the other depressions
# 2, keep all large depressions (>0.01 km2)

# there are no upstream depressions for WQ7 and WQ3 catchments so set idc_choice = 0 will extract the correct goal catchment. But for IQ1 (and perhaps other catchments)
# no upstream depression is not a valid assumption anymore, need to set idc_choice = 1.

idc_choice = 1


# Resample DEM resolution based on given resolution list
demFiles = resample_dem(input_demFile,resolutionList)

for DEMFile in demFiles:
    #set input file
    print "process " + DEMFile

    #fill
    fillRaster=outputPath+"\\"+(DEMFile.split("."))[0] + "_fill.tif"
    outFill = Fill(DEMFile)
    outFill.save(fillRaster)
    print "fill done..."

    #get depression Raster
    depressionRaster= Raster(fillRaster) - Raster(DEMFile)
    depressionRasterFile=outputPath+"\\"+(DEMFile.split("."))[0] + "_fill-DEM.tif"
    depressionRaster.save(depressionRasterFile)
    print "get depression raster done..."

    #set null
    depressionRasterSetNull=SetNull(depressionRasterFile,1,"VALUE = 0")
    depressionRasterSetNULLFile=outputPath+"\\"+(DEMFile.split("."))[0] + "_fill-DEM_setNULL.tif"
    depressionRasterSetNull.save(depressionRasterSetNULLFile)

    #get depression Shapefile
    depressionVectorFile= outputPath+"\\"+(DEMFile.split("."))[0] + "_depression.shp"
    arcpy.RasterToPolygon_conversion(depressionRasterSetNULLFile, depressionVectorFile, "NO_SIMPLIFY","VALUE")
    print "get depression vector done..."

    #add area attribute
    depressionVectorFile_withArea= outputPath+"\\"+(DEMFile.split("."))[0] + "_depression_withArea.shp"
    arcpy.CalculateAreas_stats(depressionVectorFile, depressionVectorFile_withArea)
    print "add area attribute done..."

    #get large depressions
    largeDepressionVectorFile = outputPath+"\\"+(DEMFile.split("."))[0] + "_largedepressions_001km.shp"
    arcpy.MakeFeatureLayer_management(depressionVectorFile_withArea,"lyr")
    arcpy.SelectLayerByAttribute_management("lyr","NEW_SELECTION", ' "F_AREA" > 10000 ')  #10000
    arcpy.CopyFeatures_management("lyr", largeDepressionVectorFile)
    print "get large depressions done..."

    #eliminate the multi polygon
    largeDepressionEliminationVectorFile=outputPath+"\\"+(DEMFile.split("."))[0] + "_largedepressions_001km_elimination.shp"
    arcpy.EliminatePolygonPart_management(largeDepressionVectorFile, largeDepressionEliminationVectorFile, "AREA", 5000000, "", "CONTAINED_ONLY")
    print "elimination done..."

    arcpy.MakeFeatureLayer_management(largeDepressionEliminationVectorFile, 'depressions_lyr') 
    outletFile= outputPath+"\\"+(DEMFile.split("."))[0] + "_outlet.shp"
    if idc_choice==0:
        ##ERROR: this is not correct because: if there are some upstream depressions/basins, this point cannot distinguish them and thus those basins will be merged incorrectly.
        ###this can only be used if there are no upstream depressions/basins, e.g., WQ7 and WQ3 DEMs.
        ###select the depression that contains key point. This depression is used as outlet to create simultaneous outputs
        arcpy.SelectLayerByLocation_management('depressions_lyr', 'intersect', keypointFile)
        matchcount = int(arcpy.GetCount_management('depressions_lyr')[0]) 
        if matchcount == 0:
            print('no depressions contain key point')
            continue
        else:
            arcpy.CopyFeatures_management('depressions_lyr', outletFile)
            print('outlet file created')
    elif idc_choice==1:
        arcpy.SelectLayerByLocation_management('depressions_lyr', 'intersect', eraseAreaFile,'','','INVERT') #use 'INVERT' to get the depressions that do not intersect with erase area
        arcpy.CopyFeatures_management('depressions_lyr', outletFile)
        print('outlet file created')
    else:
        arcpy.CopyFeatures_management('depressions_lyr', outletFile)
        print('outlet file created')

    #get the DEM value at the large depression
    largeDepressionDEMFile=outputPath+"\\"+(DEMFile.split("."))[0] + "_largedepressions_effective_DEM.tif"
    outExtractByMask = ExtractByMask(DEMFile, outletFile)
    outExtractByMask.save(largeDepressionDEMFile)
    print "get the DEM value at the large depression, done..."

    #set null
    largeDepressionDEMSetNull=SetNull(largeDepressionDEMFile,largeDepressionDEMFile,"VALUE = 0")
    largedepressionRasterSetNULLFile=outputPath+"\\"+(DEMFile.split("."))[0] + "_largedepressions_effective_DEM_setNULL.tif"
    largeDepressionDEMSetNull.save(largedepressionRasterSetNULLFile)
    print "large depression set null done..."

    #replace the DEM values at large depression(s) with a low elevation to create the new catchment outlet(s)
    leaveLargeDepressionsDEMFile=outputPath+"\\"+(DEMFile.split("."))[0] + "_leave_largedepressions_effective_DEM.tif"
    conRaster=Con(IsNull(largeDepressionDEMSetNull),fillRaster,1)
    conRaster.save(leaveLargeDepressionsDEMFile)
    print "replace the DEM values at large depressions with a low elevation, done..."

    #set null
    leaveLargeDepressionsDEMSetNULLFile=outputPath+"\\"+(DEMFile.split("."))[0] + "_leave_largedepressions_effective_DEM_setNULL.tif"
    setNullFile=SetNull(leaveLargeDepressionsDEMFile,leaveLargeDepressionsDEMFile,"VALUE = 0")
    setNullFile.save(leaveLargeDepressionsDEMSetNULLFile)
    print "set null done..."

    # flow direction
    flowDirectionFile=outputPath+"\\"+(DEMFile.split("."))[0] + "_leave_largedepressions_effective_DEM_setNULL_flowDirection.tif"
    outFlowDirection = FlowDirection(leaveLargeDepressionsDEMSetNULLFile, "NORMAL")
    outFlowDirection.save(flowDirectionFile)
    print "cal flow direction done..."

    # basin
    basinRasterFile=outputPath+"\\"+(DEMFile.split("."))[0] + "_leave_largedepressions_effective_DEM_setNULL_basin.tif"
    outBasin = Basin(flowDirectionFile)
    outBasin.save(basinRasterFile)
    print "cal basin done..."

    #get basin Shapefile
    basinVectorFile= outputPath+"\\"+(DEMFile.split("."))[0] + "_leave_largedepressions_effective_DEM_setNULL_basins_polygon.shp"
    basinVector=arcpy.RasterToPolygon_conversion(basinRasterFile, basinVectorFile, "NO_SIMPLIFY","VALUE")
    print "get basin vector done..."

        
    #get goal basin Shapefile
    basinFile= outputPath+"\\"+(DEMFile.split("."))[0] + "_basin.shp"
    arcpy.MakeFeatureLayer_management(basinVectorFile, 'basins_lyr') 
    arcpy.SelectLayerByLocation_management('basins_lyr', 'intersect', keypointFile)
    matchcount = int(arcpy.GetCount_management('basins_lyr')[0]) 
    if matchcount == 0:
        print('no basins contain key point')
        continue
    else:
        arcpy.CopyFeatures_management('basins_lyr', basinFile)
        print('basin file created')


    #flow accumulation
    flowAccumulationFile=outputPath+"\\"+(DEMFile.split("."))[0] + "_close_basin_flowAccumulation.tif"
    outFlowAccumulation=FlowAccumulation(outFlowDirection)
    outFlowAccumulation = ExtractByMask(outFlowAccumulation,basinFile)
    outFlowAccumulation.save(flowAccumulationFile)
    print('accumulation raster file created')

    
    basinDEMFile = outputPath+"\\"+(DEMFile.split("."))[0] + "_fill_basin.tif"
    basinDEMRaster = ExtractByMask(fillRaster,basinFile)
    basinDEMRaster.save(basinDEMFile)
    print "basin DEM calculation done..."

    curvatureFile = outputPath+"\\"+(DEMFile.split("."))[0] + "_fill_curvate.tif"
    curvatureRaster = Curvature(basinDEMRaster)
    curvatureRaster.save(curvatureFile)
    print 'curvature calculation done'

    #slope, used to calculate velocity using Manning's equation
    #this is the key step.

    #option 1, use filled DEM to calculate slope and assume pathway depresssions route meltwater downstream very quickly (NULL velocity and cost)
    slopeRaster=outputPath+"\\"+(DEMFile.split("."))[0] + "_fill_slope.tif"
    fillRaster = Con(depressionRaster==0,fillRaster,99999)  #for the pathway depressions, assume meltwater will be routed quickly downstream
    fillRaster = SetNull(fillRaster,fillRaster,"VALUE = 99999")
    outSlope = Slope(fillRaster, "PERCENT_RISE", 0.01)
    outSlope = ExtractByMask(outSlope,basinFile)
    outSlope.save(slopeRaster)
    print "slope calculation done..."


    #option 2, use original DEM to calculate slope and assume topographic depressions follow their original slopes.
    slopeRaster=outputPath+"\\"+(DEMFile.split("."))[0] + "_slope.tif"
    outSlope = Slope(DEMFile, "PERCENT_RISE", 0.01)
    outSlope = ExtractByMask(outSlope,basinFile)
    outSlope.save(slopeRaster)
    print "slope calculation done..."



    arcpy.Delete_management(depressionRaster)
    arcpy.Delete_management(depressionRasterSetNull)
    arcpy.Delete_management(depressionVectorFile)
    arcpy.Delete_management(depressionVectorFile_withArea)
    arcpy.Delete_management(largeDepressionVectorFile)
    arcpy.Delete_management(largeDepressionEliminationVectorFile)
    arcpy.Delete_management(outExtractByMask)
    arcpy.Delete_management(largeDepressionDEMSetNull)
    arcpy.Delete_management(conRaster)
    arcpy.Delete_management(setNullFile)
    arcpy.Delete_management(basinRasterFile)