#author = "Kang Yang"
#copyright = "Copyright 2018, Nanjing Greenland Hydrology Group"
#license = "GPL"
#version = "0.1"
#maintainer = "Kang Yang"
#email = "kangyang@nju.edu.cn"
#status = "Production"
#description = "plot velocity, flow length, and transport time (cumulative) histograms for different resolution DEMs"

def cm2inch(*tupl):
    inch = 2.54
    if isinstance(tupl[0], tuple):
        return tuple(i/inch for i in tupl[0])
    else:
        return tuple(i/inch for i in tupl)

def plot_density(ax,data,bins,resolution):
    #pd.DataFrame(data).plot(ax = ax, kind = 'density')
    density = gaussian_kde(data)
    #density.covariance_factor = lambda : .25
    #density._compute_covariance()
    ax.plot(bins,density(bins))


def import_raster(raster):
    ds = gdal.Open(raster)
    data = np.array(ds.GetRasterBand(1).ReadAsArray())
    NoData = ds.GetRasterBand(1).GetNoDataValue() 
    data [data == NoData] = -9999.0
    data = data[data>0]
    return data

def plot_histogram(ax,data,bins,resolution):
    weights = np.ones_like(data)/float(len(data))
    ax.hist(data, bins=bins,weights=weights,histtype='stepfilled', alpha=0.6,label = str(resolution) + ' m')
    ax.set_yticks(np.arange(0, 0.008, 0.002))

def plot_cumulative_histogram(ax,data,bins,resolution):
    ax.hist(data, bins=bins, density=True, histtype='step', linewidth=3, cumulative=-1,label = str(resolution) + ' m')


def plot_hist_cumu_resolutions(raster,bins,ax,startResolution,resolutionList,xlabel):
    data = import_raster(raster)
    ax1 = ax.twinx()
    plot_cumulative_histogram(ax,data,bins,startResolution)
    plot_histogram(ax1,data,bins,startResolution)
    idcName = raster.split('_')[-3] 
    for resolution in resolutionList:
        raster1 = raster.replace(idcName,idcName + '_'+str(resolution))
        print raster1
        data = import_raster(raster1)
        plot_cumulative_histogram(ax,data,bins,resolution)
        plot_histogram(ax1,data,bins,resolution)  
    ax.set_xlabel(xlabel)

    return ax, ax1


def return_Arnold_file_names(inputFolder,demName):
    velocityRaster = os.path.join(inputFolder,demName+'_velocity_Arnold.tif')
    flowLengthRaster = os.path.join(inputFolder,demName+'_flowlength_Arnold.tif')
    transporttimeRaster = os.path.join(inputFolder,demName+'_transportTime_Arnold.tif')
    return velocityRaster,flowLengthRaster,transporttimeRaster

import os
import sys
import math
import matplotlib
import matplotlib.pyplot as plt
from matplotlib import cm
import numpy as np
import pandas as pd
from osgeo import gdal
from pandas.io.excel import ExcelWriter
from scipy.stats import gaussian_kde
from cycler import cycler

matplotlib.style.use('classic')
matplotlib.rcParams.update({'font.size': 15})
matplotlib.rcParams['axes.linewidth'] = 2
matplotlib.rcParams['xtick.major.size'] = 15
matplotlib.rcParams['xtick.major.width'] = 2
matplotlib.rcParams['ytick.major.size'] = 15
matplotlib.rcParams['ytick.major.width'] = 2
matplotlib.rcParams['lines.linewidth'] = 4
matplotlib.rcParams['lines.markersize'] = 10
#matplotlib.rcParams['axes.prop_cycle'] = matplotlib.cycler(color=['b','r','g'])

#DEMs
#IQ1_SETSM_WV02_20140707
#WQ7_SETSM_WV01_20140919
#WQ3_SETSM_WV01_20150718
#WQ723_SETSM_WV02_20140701

demName = 'IQ1_SETSM_WV02_20140707'
inputFolder = r'D:\Meltwater_routing_modeling_new\Arnold_transport_time'
startResolution = 2
resolutionList = [90]
#resolutionList = [5,10,30,90]
idc_name = demName.split('_')[0]



velocityRaster,flowLengthRaster,transporttimeRaster = return_Arnold_file_names(inputFolder,demName)

#f, (ax1,ax2,ax3) = plt.subplots(3,figsize=(7.5,5.5))

f, (ax1,ax2,ax3) = plt.subplots(3)

bins_velocity = np.arange(0,1.0,0.001) #velocity
plot_hist_cumu_resolutions(velocityRaster,bins_velocity,ax1,startResolution,resolutionList,'velocity (m/s)')
bins_flowlength = np.arange(0,14000,25)   #flow length
ax2, ax22 = plot_hist_cumu_resolutions(flowLengthRaster,bins_flowlength,ax2,startResolution,resolutionList,'flow length (m)')
bins_transport_time = np.arange(0,70000,100)   #transpor time
plot_hist_cumu_resolutions(transporttimeRaster,bins_transport_time,ax3,startResolution,resolutionList,'transport time (s)')

ax3.legend(loc = 'upper right', fontsize = 13)
ax2.set_ylabel('cumulative fraction')
ax22.set_ylabel('fraction')



#play with different outputs and export the values you think best fit
plt.subplots_adjust(left=0.091, bottom=0.104, right=0.908, top=0.954, wspace=0.2, hspace=0.574)
plt.show()

f.savefig(os.path.join('E:',idc_name + '_compare_Arnold_variables.tif'), format='tif', dpi=450)


#top=0.954,
#bottom=0.104,
#left=0.091,
#right=0.908,
#hspace=0.574,
#wspace=0.2