#author = "Kang Yang"
#copyright = "Copyright 2018, Nanjing Greenland Hydrology Group"
#license = "GPL"
#version = "0.1"
#maintainer = "Kang Yang"
#email = "kangyang@nju.edu.cn"
#status = "Production"
#description = "plot velocity, flow length, and transport time (cumulative) histograms for different catchments"


def cm2inch(*tupl):
    inch = 2.54
    if isinstance(tupl[0], tuple):
        return tuple(i/inch for i in tupl[0])
    else:
        return tuple(i/inch for i in tupl)

def plot_density(ax,data,bins,resolution):
    #pd.DataFrame(data).plot(ax = ax, kind = 'density')
    density = gaussian_kde(data)
    #density.covariance_factor = lambda : .25
    #density._compute_covariance()
    ax.plot(bins,density(bins))


def import_raster(raster):
    ds = gdal.Open(raster)
    data = np.array(ds.GetRasterBand(1).ReadAsArray())
    NoData = ds.GetRasterBand(1).GetNoDataValue() 
    data [data == NoData] = -9999.0
    data = data[data>0]
    return data

def plot_histogram(ax,data,bins,resolution):
    weights = np.ones_like(data)/float(len(data))
    ax.hist(data, bins=bins,weights=weights,histtype='stepfilled', alpha=0.6,label = str(resolution) + ' m')
    ax.set_yticks(np.arange(0, 0.008, 0.002))

def plot_cumulative_histogram(ax,data,bins,resolution):
    ax.hist(data, bins=bins, density=True, histtype='step', linewidth=3, cumulative=-1,label = str(resolution) + ' m')


def plot_hist_cumu_resolutions(raster,bins,ax,startResolution,resolutionList):
    data = import_raster(raster)
    print '-----------'
    print np.mean(data)
    print np.std(data)
    ax1 = ax.twinx()
    plot_histogram(ax1,data,bins,startResolution)
    plot_cumulative_histogram(ax,data,bins,startResolution)
    idcName = raster.split('_')[-3] 
    for resolution in resolutionList:
        raster1 = raster.replace(idcName,idcName + '_'+str(resolution))
        data = import_raster(raster1)
        print np.mean(data)
        print np.std(data)
        plot_histogram(ax1,data,bins,resolution)  
        plot_cumulative_histogram(ax,data,bins,resolution)
    #ax.set_ylabel('\n'.join(wrap('cumulative fraction',10)))
    #ax1.set_ylabel('fraction')
    ylim = [0,0.008]
    ax1.set_ylim(ylim)


def return_Arnold_file_names(inputFolder,demName):
    velocityRaster = os.path.join(inputFolder,demName+'_velocity_Arnold.tif')
    flowLengthRaster = os.path.join(inputFolder,demName+'_flowlength_Arnold.tif')
    transporttimeRaster = os.path.join(inputFolder,demName+'_transportTime_Arnold.tif')
    return velocityRaster,flowLengthRaster,transporttimeRaster


import os
import sys
import math
import matplotlib
import matplotlib.pyplot as plt
from matplotlib import cm
import numpy as np
import pandas as pd
from osgeo import gdal
from pandas.io.excel import ExcelWriter
from scipy.stats import gaussian_kde
from textwrap import wrap
from matplotlib import cm
import string

matplotlib.style.use('classic')
matplotlib.rcParams.update({'font.size': 15})
matplotlib.rcParams['axes.linewidth'] = 2
matplotlib.rcParams['xtick.major.size'] = 15
matplotlib.rcParams['xtick.major.width'] = 2
matplotlib.rcParams['ytick.major.size'] = 15
matplotlib.rcParams['ytick.major.width'] = 2
matplotlib.rcParams['lines.linewidth'] = 4
matplotlib.rcParams['lines.markersize'] = 10
#matplotlib.rcParams['axes.prop_cycle'] = matplotlib.cycler(color=iter(cm.viridis(np.linspace(0,1,5))))
matplotlib.rcParams['axes.prop_cycle'] = matplotlib.cycler(color=['b','orange','g'])

f, (ax1,ax2,ax3,ax4) = plt.subplots(4,sharex=True,sharey=True)

axarr = []
axarr.append(ax1)
axarr.append(ax2)
axarr.append(ax3)
axarr.append(ax4)

inputFolder = r'D:\Meltwater_routing_modeling_new\Arnold_transport_time'
startResolution = 2
resolutionList = [5,90]

#DEMs
#IQ1_SETSM_WV02_20140707
#WQ7_SETSM_WV01_20140919
#WQ3_SETSM_WV01_20150718
#WQ723_SETSM_WV02_20140701


demName1 = 'IQ1_SETSM_WV02_20140707'
velocityRaster1,flowLengthRaster1,transporttimeRaster1 = return_Arnold_file_names(inputFolder,demName1)

print transporttimeRaster1

demName2 = 'WQ7_SETSM_WV01_20140919'
velocityRaster2,flowLengthRaster2,transporttimeRaster2 = return_Arnold_file_names(inputFolder,demName2)

demName3 = 'WQ3_SETSM_WV01_20150718'
velocityRaster3,flowLengthRaster3,transporttimeRaster3 = return_Arnold_file_names(inputFolder,demName3)

demName4 = 'WQ723_SETSM_WV02_20140701'
velocityRaster4,flowLengthRaster4,transporttimeRaster4 = return_Arnold_file_names(inputFolder,demName4)


bins_velocity = np.arange(0,1.0,0.001) #velocity
plot_hist_cumu_resolutions(velocityRaster1,bins_velocity,ax1,startResolution,resolutionList)
plot_hist_cumu_resolutions(velocityRaster2,bins_velocity,ax2,startResolution,resolutionList)
plot_hist_cumu_resolutions(velocityRaster3,bins_velocity,ax3,startResolution,resolutionList)
plot_hist_cumu_resolutions(velocityRaster4,bins_velocity,ax4,startResolution,resolutionList)
ax4.set_xlabel('velocity (m/s)')
ax4.legend(loc = 'upper right', fontsize = 13)


new_ax1 = f.add_subplot(121, frame_on=False)
plt.tick_params(labelcolor='none', top='off', bottom='off', left='off', right='off')
new_ax1.set_xticks([])
new_ax1.set_yticks([])
new_ax1.set_ylabel('cumulative fraction',labelpad=30)

new_ax2 = f.add_subplot(122, frame_on=False)
plt.tick_params(labelcolor='none', top='off', bottom='off', left='off', right='off')
new_ax2.set_xticks([])
new_ax2.set_yticks([])
label = new_ax2.set_ylabel('fraction',labelpad=0)
new_ax2.yaxis.set_label_coords(1.3, 0.5)



#bins_flowlength = np.arange(0,12000,25)   #flow length
#plot_hist_cumu_resolutions(flowLengthRaster,bins_flowlength,idcName,ax2,startResolution,resolutionList,'flow length (m)')
#bins_transport_time = np.arange(0,70000,100)   #transpor time
#plot_hist_cumu_resolutions(transporttimeRaster,bins_transport_time,idcName,ax3,startResolution,resolutionList,'transport time (s)')


n = 0
for ax in axarr:
    ax.text(0.02, 0.85, '(' + string.ascii_lowercase[n] + ')', transform=ax.transAxes, size=15)
    n = n + 1

#plt.tight_layout()

plt.show()

f.savefig(r'E:\compare_Arnold_variables_for_different_IDCs.tif', format='tif', dpi=450)